<?php
/**
 * @version    CVS: 1.0.0
 * @package    Com_miniorange_web3
 * @author     miniOrange Security Software Pvt. Ltd. <info@xecurify.com>
 * @copyright  Copyright 2023 miniOrange. All Rights Reserved.
 * @license    GNU/GPLv3
 */

// No direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.controllerform');
jimport('joomla.filesystem.file');


/**
 * Myaccount controller class.
 *
 * @since  1.6
 */
class Miniorange_web3ControllerMyaccount extends JControllerForm
{
    function __construct()
    {
        $this->view_list = 'myaccounts';
        parent::__construct();
    }

    function requestForTrialPlan()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if ((!isset($post['email'])) || (!isset($post['plan'])) || (!isset($post['description']))) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings');
            return;
        }
        $email = $post['email'];
        $plan = $post['plan'];
        $description = trim($post['description']);
        $demo = 'Trial';

        $customer = new Mo_web3_Local_Customer();
        $response = json_decode($customer->request_for_trial($email, $plan, $demo, $description));

        if ($response->status != 'ERROR')
        {
            $msg=($demo == 'Demo')? 'We have received your demo request. Someone from our company will contact you shortly regarding the next steps.':'We have received your trial request. Someone from our company will contact you shortly regarding the next steps.';
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings', $msg);
        }
        else {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings', 'Server is busy. Please try again later.', 'error');
            return;
        }

    }

    function verifyCustomer()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if ((!isset($post['email'])) || (!isset($post['password']))) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }

        $email = '';
        $password = '';

        if (Mo_Web3_Local_Util::check_empty_or_null($post['email']) || Mo_Web3_Local_Util::check_empty_or_null($post['password'])) {
            JFactory::getApplication()->enqueueMessage('All the fields are required. Please enter valid entries.', 'error');
            return;
        } else {
            $email = $post['email'];
            $password = $post['password'];
        }


        $customer = new Mo_web3_Local_Customer();
        $content = $customer->get_customer_key($email, $password);

        $customerKey = json_decode($content, true);

        if (strcasecmp($customerKey['apiKey'], 'CURL_ERROR') == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $customerKey['token'], 'error');
            return;
        } else if (json_last_error() == JSON_ERROR_NONE) {
            $this->saveSuccessCustomer($customerKey, 'Your account has been retrieved successfully, now you can <strong>upgrade</strong> to licensed version.', $email);
        } else {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Invalid username or password. Please try again.', 'error');
            return;
        }
    }

    function saveSuccessCustomer($customerKey, $message, $email = '')
    {
        if (empty($email)) {
            $email = isset($customerKey['email']) ? $customerKey['email'] : '';
        }
        $apiKey = isset($customerKey['apiKey']) ? $customerKey['apiKey'] : '';
        $token = isset($customerKey['token']) ? $customerKey['token'] : '';
        $phone = isset($customerKey['phone']) ? $customerKey['phone'] : '';
        $customerId = isset($customerKey['id']) ? $customerKey['id'] : '';
        $this->save_success_customer_config($email, $customerId, $apiKey, $token, $phone, $message);

    }

    function save_success_customer_config($email, $id, $apiKey, $token, $phone, $msg)
    {
        $database_name = '#__miniorange_web3_customer_details';
        $updatefieldsarray = array(
            'email' => $email,
            'customer_key' => $id,
            'api_key' => $apiKey,
            'customer_token' => $token,
            'admin_phone' => $phone,
            'login_status' => 0,
            'registration_status' => 'SUCCESS',
            'password' => '',
            'email_count' => 0,
            'sms_count' => 0,
        );

        $result = new Mo_Web3_Local_Util();
        $result->generic_update_query($database_name, $updatefieldsarray);
        $erMsg = 'Your account has been retrieved successfully, now you can <strong>upgrade</strong> to licensed version.';
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=licensing', $erMsg);
    }

    function customerLoginForm()
    {
        $database_name = '#__miniorange_web3_customer_details';
        $updatefieldsarray = array(
            'login_status' => 1,
            'password' => '',
            'sms_count' => 0,
        );

        $result = new Mo_Web3_Local_Util();
        $result->generic_update_query($database_name, $updatefieldsarray);
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
    }

    function ResetAccount()
    {
        $database_name = '#__miniorange_web3_customer_details';
        $updatefieldsarray = array(
            'customer_key' => '',
            'api_key' => '',
            'customer_token' => '',
            'admin_phone' => '',
            'login_status' => 0,
            'new_registration' => 0,
            'registration_status' => NULL,
            'email' => '',
        );

        $result = new Mo_Web3_Local_Util();
        $result->generic_update_query($database_name, $updatefieldsarray);
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'The account has been successfully removed from the plugin.');
        return;
    }

    function importexport()
    {
        $customer = new Mo_Web3_Local_Util();
        $customerResult = $customer->_load_db_values('#__miniorange_web3_config_settings');

        if (empty($customerResult['enable_web3_user_login'])) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=com_miniorange_web3&tab=config_settings', 'Please configure atleast one of the Wallet Method before exporting the configurations.', 'error');
            return;
        }
        $post = JFactory::getApplication()->input->post->getArray();
        require_once JPATH_SITE . DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . 'com_miniorange_web3' . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'export.php';

        define("Tab_Class_Names", serialize(array(
            "configure_settings" => 'mo_configure_settings',
            "proxy" => 'mo_proxy'
        )));

        $tab_class_name = unserialize(Tab_Class_Names);
        $configuration_array = array();
        foreach ($tab_class_name as $key => $value) {
            $configuration_array[$key] = $this->mo_get_configuration_array($value);
        }

        if ($configuration_array) {
            header("Content-Disposition: attachment; filename=miniorange-web3-config.json");
            echo(json_encode($configuration_array, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
            exit;
        }
        if(isset($post) && $post['test_configuration']=='true')
        {
            $this->setRedirect('index.php??morequest=acs', 'Downloded sucessfully the Web3 configuration.');
            return;
        }
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings', 'Downloded sucessfully the Web3 configuration.');
        return;
    }

    function mo_get_configuration_array($class_name)
    {
        if ($class_name == 'mo_configure_settings') {
            $customerResult = Mo_Web3_Local_Util::_get_values_from_table('#__miniorange_web3_config_settings');
        }
        if ($class_name == 'mo_proxy') {
            $customerResult = Mo_Web3_Local_Util::_get_values_from_table('#__miniorange_web3_proxy_setup');
        }

        $class_object = call_user_func($class_name . '::getConstants');
        $mo_array = array();

        foreach ($class_object as $key => $value) {
            if ($mo_option_exists = $customerResult[$value])
                $mo_array[$key] = $mo_option_exists;
        }
        return $mo_array;
    }

    function saveAdminMail()
    {
        $post=	JFactory::getApplication()->input->post->getArray();
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $fields = array(
            $db->quoteName('email') . ' = '.$db->quote($post['admin_email']),

        );

        $conditions = array(
            $db->quoteName('id') . ' = 1'
        );

        $query->update($db->quoteName('#__miniorange_web3_customer_details'))->set($fields)->where($conditions);
        $db->setQuery($query);
        $result = $db->execute();
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=com_miniorange_web3&tab=config_settings','Email successfully changed');
        return;
    }

    function saveConfig()
    {
        $dest_file = JPATH_ROOT . "/ajax.php";
        if (!file_exists($dest_file)) {
            Mo_Web3_Local_Util::create_ajax_file($dest_file);
        }

        $post = JFactory::getApplication()->input->post->getArray();

        if (count($post) == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings');
            return;
        }

        $wallet_array = $post;

        $web3_user_login_array = array();
        foreach ($wallet_array as $key => $value)
        {
            if($key != 'option1')
            {
                $web3_user_login_array[$key] = $value;
            }
        }

        $database_name = '#__miniorange_web3_config_settings';
        $updatefieldsarray = array(
                'enable_web3_user_login'        => json_encode($web3_user_login_array),
        );

        $result = new Mo_Web3_Local_Util();
        $result->generic_update_query($database_name, $updatefieldsarray);

        //Save web3 configuration
        $message = 'Your configuration has been saved successfully. Please click on <a href="index.php?option=com_miniorange_web3&tab=config_settings&test-config=true" class="mo_boot_btn mo_boot_btn-saml">Test Configuration</a> button to proceed futher.';
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=config_settings', $message);
    }

    function registerCustomer()
    {
        //validate and sanitize
        $post = JFactory::getApplication()->input->post->getArray();
        if (count($post) == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }
        $email = '';
        $phone = '';
        $password = '';
        $confirmPassword = '';
        if (Mo_Web3_Local_Util::check_empty_or_null($post['email']) || Mo_Web3_Local_Util::check_empty_or_null($post['password']) || Mo_Web3_Local_Util::check_empty_or_null($post['confirmPassword'])) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'All the fields are required. Please enter valid entries.', 'error');
            return;
        } else if (strlen($post['password']) < 6 || strlen($post['confirmPassword']) < 6) {    //check password is of minimum length 6
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Choose a password with minimum length 6.', 'error');
            return;
        } else {
            $email = $post['email'];
            $phone = $post['phone'];
            $password = $post['password'];
            $confirmPassword = $post['confirmPassword'];
        }

        $database_name = '#__miniorange_web3_customer_details';
        $updatefieldsarray = array(
            'email' => $email,
        );

        $proxy_setup = new Mo_Web3_Local_Util();
        $proxy_setup->generic_update_query($database_name, $updatefieldsarray);

        if ($phone != '') {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array(
                'admin_phone' => $phone,
            );

            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
        }

        if (strcmp($password, $confirmPassword) == 0) {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array(
                'password' => base64_encode($password),
            );

            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);

            $customer = new Mo_web3_Local_Customer();
            $content = json_decode($customer->check_customer($email), true);
            if (strcasecmp($content['status'], 'CUSTOMER_NOT_FOUND') == 0) {
                $auth_type = 'EMAIL';
                $content = json_decode($customer->send_otp_token($auth_type, null), true);
                if (strcasecmp($content['status'], 'SUCCESS') == 0) {
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array(
                        'email_count' => '1',
                        'transaction_id' => $content['txId'],
                        'registration_status' => 'MO_OTP_DELIVERED_SUCCESS',
                    );

                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'A One Time Passcode(OTP) has been sent to <strong>' . $email . '</strong>. Please enter the OTP to verify your email. ');
                } else {
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array(
                        'registration_status' => 'MO_OTP_DELIVERED_FAILURE',
                    );

                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'There is an error in sending OTP. Please click on Resend OTP to try again. ', 'error');
                    return;
                }
            } else if (strcasecmp($content['status'], 'CURL_ERROR') == 0) {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'registration_status' => 'MO_OTP_DELIVERED_FAILURE',
                );

                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $content['statusMessage'], 'error');
                return;
            } else if (strcasecmp($content['status'], 'TRANSACTION_LIMIT_EXCEEDED') == 0) {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'login_status' => 1,
                    'registration_status' => '',
                );

                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Server is busy. Please contact us at joomlasupport@xecurify.com or <a href=\'http://miniorange.com/contact\' target=\'_blank\'>click here</a> to contact us for support. We will reach back to you! ', 'error');
                return;
            } else {
                $content = $customer->get_customer_key($email, $password);
                $customerKey = json_decode($content, true);
                if (json_last_error() == JSON_ERROR_NONE) {
                    $this->saveSuccessCustomer($customerKey, 'Your account has been retrieved successfully, now you can <strong>upgrade</strong> to licensed version.', $email);

                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array('password' => '',);
                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                } else {
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array(
                        'login_status' => 1,
                        'new_registration' => 0,
                        'password' => '',
                    );

                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'You already have an account with miniOrange. Please enter a valid password.', 'error');
                    return;
                }
            }
        } else {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array('login_status' => 0,);

            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Password and Confirm password do not match.', 'error');
            return;
        }
    }

    function validateOtp()
    {
        //validation and sanitization
        $post = JFactory::getApplication()->input->post->getArray();

        if (!isset($post['otp_token'])) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }
        $otp_token = '';

        if (Mo_Web3_Local_Util::check_empty_or_null($post['otp_token'])) {

            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array('registration_status' => 'MO_OTP_VALIDATION_FAILURE',);
            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Please enter a value in otp field.', 'error');
            return;
        } else {
            $otp_token = trim($post['otp_token']);
        }

        $transaction_detail = new Mo_Web3_Local_Util();
        $transaction_id = $transaction_detail->_load_db_values('#__miniorange_web3_customer_details');

        $transaction_id = $transaction_id['transaction_id'];

        $customer = new Mo_web3_Local_Customer();


        $content = json_decode($customer->validate_otp_token($transaction_id, $otp_token), true);

        if (strcasecmp($content['status'], 'SUCCESS') == 0) {
            $customer = new Mo_web3_Local_Customer();
            $customerKey = json_decode($customer->create_customer(), true);
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array(
                'email_count' => 0,
                'sms_count' => 0,
            );
            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            if (strcasecmp($customerKey['status'], 'CUSTOMER_USERNAME_ALREADY_EXISTS') == 0) {    //admin already exists in miniOrange
                $content = $customer->get_customer_key($transaction_id['email'], $transaction_id['password']);
                $customerKey = json_decode($content, true);
                if (json_last_error() == JSON_ERROR_NONE) {
                    $this->saveSuccessCustomer($customerKey, 'Your account has been created successfully.', '');
                } else {
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array(
                        'registration_status' => 'false',
                        'login_status' => 1,
                        'password' => '',
                    );
                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'You already have an account with miniOrange. Please enter a valid password.', 'error');
                    return;
                }
            } else if (strcasecmp($customerKey['status'], 'INVALID_EMAIL') == 0) {
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Invalid Email ID. Please use a valid Email ID to register .', 'error');
                return;
            } else if (strcasecmp($customerKey['status'], 'SUCCESS') == 0) {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'password' => '',
                );
                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);

                //registration successful
                $this->saveSuccessCustomer($customerKey, 'Registration complete!', '');
            } else if (strcmp($customerKey['message'], 'Email is not enterprise email.') == 0) {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'registration_status' => '',
                    'email' => '',
                    'password' => '',
                    'transaction_id' => '',
                );
                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'There was an error creating an account for you. You may have entered an invalid Email-Id. <br><strong>(We discourage the use of disposable emails)</strong><br>
											Please try again with a valid email.', 'error');
                return;
            } else {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'registration_status' => '',
                    'email' => '',
                    'password' => '',
                    'transaction_id' => '',
                );
                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'There was an error creating an account for you. You may have entered an invalid Email-Id. <br><strong>(We discourage the use of disposable emails)</strong><br>
											Please try again with a valid email.', 'error');
                return;
            }
        } else if (strcasecmp($content['status'], 'INVALID_EMAIL_QUICK_EMAIL') == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'There was an error creating an account for you.
                            You may have entered an invalid Email-Id. <br><strong>(We discourage the use of disposable emails)</strong><br>Please go back and try again with a valid email.', 'error');
            return;
        } else if ($content['status'] == 'FAILED') {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'An error occured, while processing your request.', 'error');
            return;
        } else if (strcasecmp($content['status'], 'CURL_ERROR') == 0) {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array(
                'registration_status' => 'MO_OTP_VALIDATION_FAILURE',
            );

            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $content['statusMessage'], 'error');
            return;
        } else {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array(
                'registration_status' => 'MO_OTP_VALIDATION_FAILURE',
            );

            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Invalid one time passcode(OTP). Please enter a valid OTP.', 'error');
            return;
        }
    }

    function resendOtp()
    {
        $customer = new Mo_web3_Local_Customer();
        $auth_type = 'EMAIL';
        $content = json_decode($customer->send_otp_token($auth_type, null), true);
        if (strcasecmp($content['status'], 'SUCCESS') == 0) {
            $customer_details = (new Mo_Web3_Local_Util)->_load_db_values('#__miniorange_web3_customer_details');
            $email_count = $customer_details['email_count'];
            $admin_email = $customer_details['email'];
            if ($email_count != '') {
                $email_count = $email_count + 1;
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array(
                    'email_count' => $email_count,
                    'transaction_id' => $content['txId'],
                    'registration_status' => 'MO_OTP_DELIVERED_SUCCESS',
                );

                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Another One Time Passcode has been sent to <strong>' . ($admin_email) . '</strong>. Please enter the OTP below to verify your email.');
            } else {
                $database_name = '#__miniorange_web3_customer_details';
                $updatefieldsarray = array('registration_status' => 'MO_OTP_DELIVERED_SUCCESS',);
                $proxy_setup = new Mo_Web3_Local_Util();
                $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'An OTP has been sent to <strong>' . ($admin_email) . '</strong>. Please enter the OTP below to verify your email.');
            }
        } else if (strcasecmp($content['status'], 'CURL_ERROR') == 0) {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array('registration_status' => 'MO_OTP_DELIVERED_FAILURE',);
            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $content['statusMessage'], 'error');
            return;
        } else {
            $database_name = '#__miniorange_web3_customer_details';
            $updatefieldsarray = array('registration_status' => 'MO_OTP_DELIVERED_FAILURE',);
            $proxy_setup = new Mo_Web3_Local_Util();
            $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'There is an error in sending OTP. Please click on Resend OTP to try again.', 'error');
            return;
        }
    }

    function cancelform()
    {
        $database_name = '#__miniorange_web3_customer_details';
        $updatefieldsarray = array('email' => '',
            'registration_status' => '',
            'login_status' => 0,
            'email_count' => 0,
            'sms_count' => 0,
        );

        $proxy_setup = new Mo_Web3_Local_Util();
        $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
    }

    function phoneVerification()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if (count($post) == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }
        $phone = $post['phone_number'];
        $phone = str_replace(' ', '', $phone);
        $pattern = "/[\+][0-9]{1,3}[0-9]{10}/";
        if (preg_match($pattern, $phone, $matches, PREG_OFFSET_CAPTURE)) {
            $auth_type = 'SMS';
            $customer = new Mo_web3_Local_Customer();
            $send_otp_response = json_decode($customer->send_otp_token($auth_type, $phone));
            if ($send_otp_response->status == 'SUCCESS') {
                $sms_count = new Mo_Web3_Local_Util();
                $sms_count = $sms_count->_load_db_values('#__miniorange_web3_customer_details');
                $sms_count = $sms_count['sms_count'];
                if ($sms_count != '') {
                    $sms_count = $sms_count + 1;
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array('sms_count' => $sms_count,
                        'transaction_id' => $send_otp_response->txId,
                    );
                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Another One Time Passcode has been sent <strong>(' . $sms_count . ')</strong> for verification to ' . $phone);
                } else {
                    $database_name = '#__miniorange_web3_customer_details';
                    $updatefieldsarray = array('sms_count' => 1,
                        'transaction_id' => $send_otp_response->txId,
                    );
                    $proxy_setup = new Mo_Web3_Local_Util();
                    $proxy_setup->generic_update_query($database_name, $updatefieldsarray);
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'One Time Passcode has been sent ( <strong>1</strong> ) for verification to ' . $phone);
                }
            }
        } else {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Please enter the phone number in the following format: <strong>+##country code## ##phone number##', 'error');
            return;
        }
    }

    function contactUs()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if (count($post) == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }
        if (isset($post['query_phone']) && $post['query_phone'] != NULL) {
            $pgone_num_validate = preg_match("/^\+?[0-9]+$/", $post['query_phone']);
            if (!$pgone_num_validate) {
                $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Please submit a valid phone number.', 'error');
                return;
            }
        }

        if (Mo_Web3_Local_Util::check_empty_or_null($post['query_email'])) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Please submit your query with email.', 'error');
            return;
        } else if (Mo_Web3_Local_Util::check_empty_or_null(trim($post['mo_saml_query_support'] || trim($post['mo_saml_query_support'])))) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Query cannot be empty.', 'error');
            return;
        } else {
            $query = $post['mo_saml_query_support'];
            $email = $post['query_email'];
            $phone = $post['query_phone'];
            $contact_us = new Mo_web3_Local_Customer();
            $submited = json_decode($contact_us->submit_contact_us($email, $phone, $query), true);
            if (json_last_error() == JSON_ERROR_NONE) {
                if (is_array($submited) && array_key_exists('status', $submited) && $submited['status'] == 'ERROR') {
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $submited['message'], 'error');
                    return;
                } else {
                    if ($submited == false) {
                        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Your query could not be submitted. Please try again.', 'error');
                        return;
                    } else {
                        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Thanks for getting in touch! We shall get back to you shortly.');
                    }
                }
            }
        }
    }

    function callContactUs()
    {
        $post = JFactory::getApplication()->input->post->getArray();
        if (count($post) == 0) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account');
            return;
        }
        $query_email = $post['mo_sp_setup_call_email'];
        $query = $post['mo_sp_setup_call_issue'];
        $description = $post['mo_sp_setup_call_desc'];
        $callDate = $post['mo_sp_setup_call_date'];
        $timeZone = $post['mo_sp_setup_call_timezone'];
        if ($this->checkEmptyOrNull($timeZone) || $this->checkEmptyOrNull($callDate) || $this->checkEmptyOrNull($query_email) || $this->checkEmptyOrNull($query) || $this->checkEmptyOrNull($description)) {
            $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Mentioned fields are mandatory', 'error');
            return;
        } else {
            $contact_us = new Mo_web3_Local_Customer();
            $submited = json_decode($contact_us->request_for_setupCall($query_email, $query, $description, $callDate, $timeZone), true);
            if (json_last_error() == JSON_ERROR_NONE) {
                if (is_array($submited) && array_key_exists('status', $submited) && $submited['status'] == 'ERROR') {
                    $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', $submited['message'], 'error');
                } else {
                    if ($submited == false) {
                        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Your query could not be submitted. Please try again.', 'error');
                    } else {
                        $this->setRedirect('index.php?option=com_miniorange_web3&tab=account', 'Thanks for getting in touch! We shall get back to you shortly.');
                    }
                }
            }

        }
    }

    function checkEmptyOrNull($value)
    {
        if (!isset($value) || empty($value)) {
            return true;
        }
        return false;
    }
}
